using System;
using System.Collections;

namespace EmitCode
{
    /// <summary>
    /// Filtered collection
    /// </summary>
    public class FilteredCollection : IEnumerable
    {
        public Array ToArray()
        {
            ArrayList list = new ArrayList();
            foreach (Object elem in collection) 
            { 
                if (condition.Evaluate(elem)) 
                { 
                    list.Add(elem);
                }
            }
            return list.ToArray(elemType);
        }

        private class FilteredEnumerator : IEnumerator 
        {
            private IEnumerator enumerator;
            private ICondition  condition;
            private Object curr;

            internal FilteredEnumerator(ICollection collection, ICondition condition) 
            { 
                enumerator = collection.GetEnumerator();
                this.condition = condition;
            }

            public void Reset() 
            { 
                enumerator.Reset();
            }

            public bool MoveNext() 
            {
                while (enumerator.MoveNext()) 
                {
                    curr = enumerator.Current;
                    if (condition.Evaluate(curr)) 
                    {
                        return true;
                    }
                }
                return false;
            }
            public Object Current 
            {
                get 
                {
                    return curr;
                }
            }
        }

        public IEnumerator GetEnumerator() 
        {
            return new FilteredEnumerator(collection, condition);
        }

        internal FilteredCollection(ICollection collection, Type elemType, ICondition condition)
        {
            this.collection = collection;
            this.elemType = elemType;
            this.condition = condition;
        }

        private ICollection collection;
        private Type        elemType;
        private ICondition  condition;
    }
}
